import os

import numpy as np
from shutil import copyfile

from . import Christoffel
from . import ElasticConstants as ECs
from . import GetDOS as dos
from . import ReadData as data
from . import modematch
from . import qha2


class Control:
	def __init__(self,x,c,a):
		self.xtal = x
		self.count = c
		self.atoms = a
	def Modematch(self):
		print("Shifting ", self.xtal, " Frequencies...")
		CurrentDir = os.getcwd()
		XtalPath = CurrentDir + os.path.sep + self.xtal + os.path.sep
		AllFreqs = []
		for x in range(self.count):
			x = str(x+1)
			SubDir = XtalPath + x
			
			SSFreqFile = SubDir + os.path.sep + self.xtal + x + '.ss.yaml'
			RefFreqFile = SubDir + os.path.sep + self.xtal + x + '.ref.yaml'
			ShiftFreqFile = SubDir + os.path.sep + self.xtal + x + '.shift.yaml'

			Supercell_Params = data.ReadYaml(SSFreqFile)
			[ss_freqs, mesh] = Supercell_Params.get_SS()
			shifted_freqs  = modematch.Match(self.atoms,ss_freqs,RefFreqFile,ShiftFreqFile)
			np.savetxt(self.xtal + x + 'freqs.csv', shifted_freqs)
			AllFreqs = np.append(AllFreqs, shifted_freqs)
		
		AllFreqs = np.reshape(AllFreqs,(self.count,-1))
		return AllFreqs, mesh
		
	def MakePaths(self):
		CurrentDir = os.getcwd()
		RawfileDir = CurrentDir + os.path.sep + 'datafiles' + os.path.sep
		XtalPath = CurrentDir + os.path.sep + self.xtal + os.path.sep
		if not os.path.exists(XtalPath):
			os.mkdir(XtalPath)
		for file in os.listdir(RawfileDir):
			if file.startswith(self.xtal):
				try:
					source = RawfileDir + file
					target = XtalPath + file
					copyfile(source, target)
				except OSError as e:
					print('No File %s' % e)
					continue

		for x in range(self.count):
			x = str(x+1)
			SubDir = XtalPath + x
			if not os.path.exists(SubDir):
				os.mkdir(SubDir)
			SSFreqFile = XtalPath + self.xtal + x + '.ss.yaml'
			RefFreqFile = XtalPath + self.xtal + x + '.ref.yaml'
			ShiftFreqFile = XtalPath + self.xtal + x + '.shift.yaml'

			SSFreqFileOut = SubDir + os.path.sep + self.xtal + x + '.ss.yaml'
			RefFreqFileOut = SubDir + os.path.sep + self.xtal + x + '.ref.yaml'
			ShiftFreqFileOut = SubDir + os.path.sep + self.xtal + x + '.shift.yaml'

			try:
				os.rename(SSFreqFile, SSFreqFileOut)
			except FileExistsError:
				os.remove(SSFreqFileOut)
				os.rename(SSFreqFile, SSFreqFileOut)

			try:
				os.rename(RefFreqFile, RefFreqFileOut)
			except FileExistsError:
				os.remove(RefFreqFileOut)
				os.rename(RefFreqFile, RefFreqFileOut)

			try:
				os.rename(ShiftFreqFile, ShiftFreqFileOut)
			except FileExistsError:
				os.remove(ShiftFreqFileOut)
				os.rename(ShiftFreqFile, ShiftFreqFileOut)

	def SolveECs(self):
		print("Solving ", self.xtal, " Elastic Constants...")
		CurrentDir = os.getcwd()
		XtalPath = CurrentDir + os.path.sep + self.xtal + os.path.sep
		AllConstants = []
		for x in range(self.count):
			x = str(x + 1)
			SubDir = XtalPath + x

			StrainFile = XtalPath + self.xtal + '.strains'
			StressFile = XtalPath + self.xtal + x + '.stresses'

			#StrainFileOut = SubDir + os.path.sep + self.xtal + '.strains'
			StressFileOut = SubDir + os.path.sep + self.xtal + x + '.stresses'

			#try:
			#	os.rename(StrainFile,StrainFileOut)
			#except FileExistsError:
			#	os.remove(StrainFileOut)
			#	os.rename(StrainFile,StrainFileOut)
			try:
				os.rename(StressFile,StressFileOut)
			except FileExistsError:
				os.remove(StressFileOut)
				os.rename(StressFile,StressFileOut)

			EC_Matrix = ECs.ElasticConstants(StressFileOut,StrainFile)
			ECFile = XtalPath + self.xtal + x + '.ecs'
			np.savetxt(ECFile, EC_Matrix, delimiter='\t')
			EC_Matrix = EC_Matrix.flatten()
			AllConstants = np.append(AllConstants, EC_Matrix)
	
		AllConstants = np.reshape(AllConstants, (self.count,-1))
		return AllConstants

	def ReadECs(self):
		print("Found Elastic constants, solving dispersion...")
		CurrentDir = os.getcwd()
		XtalPath = CurrentDir + os.path.sep + self.xtal + os.path.sep
		AllConstants = []

		for x in range(self.count):
			x = str(x + 1)

			ECFile = XtalPath + self.xtal + x + '.ecs'
			ECFileOut = XtalPath + os.path.sep + x + os.path.sep + self.xtal + x + '.ecs'

			try:
				copyfile(ECFile,ECFileOut)
				os.remove(ECFile)
			except OSError as e:
				print('Cannot Find file %s' % e)
				continue

			file = open(ECFileOut, "r")
			ECs = []
			for line in file:
				ECs = np.append(ECs, np.double(line.split()))
			file.close()
			AllConstants = np.append(AllConstants,ECs)
		AllConstants = AllConstants.reshape([self.count,-1])
		return AllConstants



	def Dispersion(self,ECs):
		CurrentDir = os.getcwd()
		XtalPath = CurrentDir + os.path.sep + self.xtal + os.path.sep
		AllFreqs = []
		for x in range(self.count):	
			y = str(x + 1)
			SubDir = XtalPath + y

			SSFreqFile = SubDir + os.path.sep + self.xtal + y + '.ss.yaml'
			ShiftedFreqFile = SubDir + os.path.sep + self.xtal + y + '.freqs'
			Supercell_Params = data.ReadYaml(SSFreqFile)
			lattice = Supercell_Params.get_lattice()

			InputECs = ECs[x]
			NewFreqs = Christoffel.ECAcoustics(self.atoms,SSFreqFile,InputECs,lattice)

			np.savetxt(ShiftedFreqFile, NewFreqs)
			AllFreqs = np.append(AllFreqs, NewFreqs)
		
		AllFreqs = np.reshape(AllFreqs,(self.count,-1))
		return AllFreqs

	def ShiftPlusECs(self,Freqs):
		print("Shift + EC Correction...")
		CurrentDir = os.getcwd()  #+ os.path.sep + 'datafiles'
		XtalPath = CurrentDir + os.path.sep + self.xtal + os.path.sep
		AllFreqs = []
		for x in range(self.count):
			y = str(x + 1)
			SubDir = XtalPath + y
			RefFreqFile = SubDir + os.path.sep + self.xtal + y  + '.ref.yaml'
			ShiftFreqFile = SubDir + os.path.sep + self.xtal + y + '.shift.yaml'
			ShiftedFreqFile = SubDir + os.path.sep + self.xtal + y + '.freqs'
			ss_freqs = Freqs[x,:]
			NewFreqs = modematch.Match(self.atoms,ss_freqs,RefFreqFile,ShiftFreqFile)

			np.savetxt(ShiftedFreqFile, NewFreqs)
			AllFreqs = np.append(AllFreqs, NewFreqs)

		AllFreqs = np.reshape(AllFreqs,(self.count,-1))
		return AllFreqs

	def QuasiHarmonic(self,Freqs,Vols,Press):
		CurrentDir = os.getcwd()
		XtalPath = CurrentDir + os.path.sep + self.xtal + os.path.sep
		CurveFile = XtalPath + self.xtal + '.dat'
		EVParams = data.Quasiharmonic(CurveFile)
		ev_data = EVParams.get_evcurve()
		Vols = np.double(Vols)
		if np.size(Vols) == 1:
			AllData = dos.EvaluateDOS(Freqs,self.atoms)
		else:
			AllData = qha2.QHA(Freqs,Vols,ev_data,self.count,self.atoms,Press)
		return AllData
		
	def PhaseTransControl(self,AllData1,AllData2,Press):
		PhaseTransData = qha2.PhaseTrans(AllData1,AllData2,Press)
		return PhaseTransData
		
	



import numpy as np

def EvaluateDOS(ssfreqs, natoms):
	ThzToCm = 33.35640952
	Na = 6.0221367e23
	kb = 1.3806488e-23
	R = kb*Na
	dim = 3 * natoms

	w_0 = ssfreqs.flatten() * ThzToCm
	ssfreqs = np.reshape(ssfreqs,(dim,-1)) * ThzToCm

	use_imaginary = 0
	if use_imaginary == 0:
		for i in range(np.size(w_0)):
			if w_0[i] < 0:
				w_0[i] = 0
	
	#Create xrange
	xS = np.amin(w_0) - 100#Get Dist +- 100 from freq range in ssfreqs
	xL = np.amax(w_0) + 100
	grid = int(dim * (np.size(ssfreqs) /(dim)) + 200) #100 for xS  + xL
	x = np.linspace(xS,xL,grid)
	
	#find avg dx
	dx = []
	for i in np.arange(np.size(x) - 1):
		tmp = x[i + 1] - x[i]		
		dx = np.append(dx,tmp)
	dx = np.mean(dx)

	#Evaluate PDF in xrange. bw_method is bw; change if necessary
	y = np.zeros(grid)
	bw = 5 #in wavenumbers ; this is consistent with the std / 20 that NyDay 2016 used (Just for all modes)
	for i in np.arange(dim):
		band = ssfreqs[i,:]
		for j in np.arange(np.size(band)):
			mean = np.mean(band)
			g_x = np.exp(-np.square(x - mean) / (2  * np.square(bw)))
			y = y + g_x

	#Normalized Density of States 
	y_0 = y / (np.size(x) * bw * np.sqrt(2 * np.pi))

	#Create plot
	#axes = plt.gca()
	#axes.set_xlim([xS,xL])
	#axes.set_ylim([0,np.amax(y_0)+0.001])

	#Check norm; print A_0 if needed
	A_0 = 0
	for i in range(np.size(x) - 1):
		A_0 = A_0 + y_0[i] * dx

	tL = 500
	tS = 0
	T = np.arange(tS,tL,10)
	FvibArray = []
	HvibArray = []
	SvibArray = []

	for temp in T:
		thermo = Thermo(x,temp)
	
	##FVIB##	
		Fw_0 = thermo.Helmholtz() * y_0
		Fw_0[0] = 0
		F_vib = 0

		for i in range(np.size(x)):
			flag = np.isfinite(Fw_0[i])
			if flag == True:
				F_vib = F_vib + Fw_0[i] * dx

		F_vib = 3 * natoms * Na * F_vib / 1000
		FvibArray = np.append(FvibArray, F_vib)
	
	##ENTHALPY##
		Hw_0 = thermo.Enthalpy() * y_0
		Hw_0[0] = 0
		H_vib = 0
	
		for i in range(np.size(x)):
			flag = np.isfinite(Hw_0[i])
			if flag == True:
				H_vib = H_vib + Hw_0[i] * dx
	
		H_vib = 3 * Na * natoms * H_vib / 1000
		HvibArray = np.append(HvibArray, H_vib)
	
	##ENTROPY##
		Sw_0 = thermo.Entropy() * y_0
		Sw_0[0] = 0
		Svib = 0
	
		for i in range(np.size(x)):
			flag = np.isfinite(Sw_0[i])
			if flag == True:
				Svib = Svib + Sw_0[i] * dx

		Svib = 3 * R * Svib * natoms # // Fultz 2009
		SvibArray = np.append(SvibArray, Svib)

	#AllData = pd.DataFrame({'Temperature': np.array(T),
                                    #'Hvib': np.array(HvibArray),
                                    #'Svib': np.array(SvibArray),
                                    #'Fvib': np.array(FvibArray)})


	return FvibArray, HvibArray, SvibArray, T

class Thermo:
	def __init__(self, x, temp):
		self.T = temp
		self.h = 6.62607363e-34
		self.Na = 6.0221367e23
		self.kb = 1.3806488e-23
		self.c = 2.99792458e8
		self.w = x * (self.c * 100)
	def Helmholtz(self):
		if self.T == 0:
			F_fun = self.w * self.h / 2
		else:
			F_fun = self.kb * self.T * np.log(2*np.sinh(self.h * self.w / (2 * self.kb * self.T)))
		return F_fun

	def Enthalpy(self):
		if self.T == 0:
			H_fun = self.w * self.h / 2
		else:
			H_fun = self.h / 2 * self.w * (np.cosh(self.h * self.w / (2 * self.kb * self.T)) / np.sinh(self.h * self.w / (2 * self.kb * self.T)))
		return H_fun

	def Entropy(self):
		eps = self.w * self.h #DOS in energy
		occ = 1 / (np.exp(eps / (self.kb * self.T)) - 1 ) #Planck Distribution // Fultz 2009
		S_fun = (occ + 1) * np.log(occ + 1) - occ * np.log(occ) #Svib for phonons
		return S_fun

	def HeatCap(self):
		Cv_fun = self.kb * np.square((self.h * self.w / (2 * self.kb * self.T))) * np.square(1 / (np.sinh(self.h * self.w / (2 * self.kb * self.T))))
		return Cv_fun



from . import ReadData as data
import numpy as np
import warnings
from matching.games import StableMarriage

def get_dots(dim,shift,ref):
	dotprods = np.array([])
	for i in range(0,dim):
		for j in range(0,dim):
			val = np.dot(ref[i],shift[j])
			val = abs(val)
			dotprods = np.append(dotprods,val)
	dotprods = np.reshape(dotprods,(dim,dim))
	dotprods = np.transpose(dotprods)
	return dotprods

def Match(atom_ID,ss_freqs,RefFile,ShiftFile):
	warnings.filterwarnings("ignore")

	#All Atom Info
	natoms = sum(atom_ID)
	dim = natoms * 3
	ss_freqs = np.reshape(ss_freqs,(-1,dim))

	Ref_UCParams = data.ReadYaml(RefFile)
	[ref_freqs, ref_vecs] = Ref_UCParams.get_UC()
	ref_vecs = np.reshape(ref_vecs,(-1,dim))
	ref_freqs[0] = 0
	ref_freqs[1] = 0
	ref_freqs[2] = 0

	Shift_UCParams = data.ReadYaml(ShiftFile)
	[shift_freqs, shift_vecs] = Shift_UCParams.get_UC()
	shift_vecs = np.reshape(shift_vecs,(-1,dim))

	#Get initial dotprod matrix
	dots = get_dots(dim,shift_vecs,ref_vecs)

	dft_ranks = [] #Ranking DFT matches with FIXED DFTB modes
	for i in range(dim):
		sort_dots = np.sort(dots[i,:])[::-1]
		seen = set()
		available_matches = list(range(0,dim))
		for j in range(dim):
			val = sort_dots[j]
			arr = dots[i,:]
			tmp = (np.where(arr == val)[0][0])
			if tmp not in seen:
				available_matches.remove(tmp)
				seen.add(tmp)
			else:
				availdots = []
				for k in available_matches:
					availdots.append(dots[i,k])
				tmp = available_matches[availdots.index(max(availdots))]
				available_matches.remove(tmp)
			dft_ranks = np.append(dft_ranks,tmp)

	dftb_ranks = [] #Ranking DFTB matches with FIXED DFT modes
	for i in range(dim):
		sort_dots = np.sort(dots[:,i])[::-1]
		seen = set()
		available_matches = list(range(0,dim))
		for j in range(dim):
			val = sort_dots[j]
			arr = dots[:,i]
			tmp = (np.where(arr == val)[0][0])
			if tmp not in seen:
				available_matches.remove(tmp)
				seen.add(tmp)
			else:
				availdots = []
				for k in available_matches:
					availdots.append(dots[k,i])
				tmp = available_matches[availdots.index(max(availdots))]
				available_matches.remove(tmp)
			dftb_ranks = np.append(dftb_ranks,tmp)

	dft_ranks = dft_ranks.reshape([-1,dim])
	dftb_ranks = dftb_ranks.reshape([-1,dim])

	dftRankDict = dict(((i), dft_ranks[i][:]) for i in range(dim)) #Favored DFT pairing to a DFTB mode -- "Suitor"
	dftbRankDict = dict(((i), dftb_ranks[i][:]) for i in range(dim)) #Favored DFTB pairing to a DFT mode -- "Reviewer"
	
	matching = StableMarriage.create_from_dictionaries(dftRankDict, dftbRankDict)
	AllMatches = matching.solve()
	AllMatches = list(str(AllMatches).replace('{','').replace('}','').split(','))
	matches = []

	for i in AllMatches:
		line = (i.split(':'))
		DFTmode  = int(line[0])
		DFTBmode = int(line[1])
		matches.append(DFTmode)
		matches.append(DFTBmode)

	matches = np.array(matches).reshape([dim,2])

	##In matches array, the first column is the DFT modes, and the second is DFTB. Need to re-sort in order to get the
	##DFTB modes in ascending order for the shift
	match_array = matches[np.argsort(matches[:,1])]
	match_array = match_array[:,0]
	#Set new order according to match
	shift_freqs = shift_freqs[match_array]

	#Apply shifts to dftb freqs
	shifts = [ref_freqs - shift_freqs]
	shifts = np.array(shifts).reshape(dim,-1)

	ss_freqs = ss_freqs.transpose() 
	shifted_freqs = ss_freqs[match_array] + shifts
	shifted_freqs = shifted_freqs.flatten()
	

	
	
	return shifted_freqs




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
	long_description = fh.read()

setuptools.setup(
	name = "modematch",
	version = "0.0.7",
	author="Cameron Cook",
	author_email="cam.cook32@gmail.com",
	description="Crystal free energy prediction suite",
	long_discription=long_description,
	long_description_content_type="text/markdown",
	packages=setuptools.find_packages(),
	install_requires=[
		'numpy',
		'sympy',
		'pandas',
		'matching',
		],
	url="https://github.com/cjcook41/Modematching.git",
	classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
	],
	python_requires='>=3.7',
)



from . import ReadData as data
import numpy as np
import cmath

def ECAcoustics(atom_IDs,SSFreqFile,ECs,lattice):
		dim = sum(atom_IDs) * 3
		bohr2ang = 0.529177 ## YAML files start in bohr
		ss_params = data.ReadYaml(SSFreqFile)
		mesh = ss_params.get_SS()[1]
		lattice = lattice.reshape([3,3]) * bohr2ang

		##IN ANGSTROM
		a = lattice[0,:]
		b = lattice[1,:]
		c = lattice[2,:]

		#a_norm = np.linalg.norm(a)
		#b_norm = np.linalg.norm(b)
		#c_norm = np.linalg.norm(c)

		#beta = np.arccos(np.dot(a,c) / (a_norm * c_norm))
		#alpha = np.arccos(np.dot(b,c) / (b_norm * c_norm))
		#gamma = np.arccos(np.dot(a,b) / (a_norm * b_norm))

		v_cross = np.cross(b,c)
		v_dot = np.dot(a,v_cross)
		volume = v_dot * np.power(1E-10,3) 

		#Get the reciprocal Lattice
		recip_lattice = []
		astar = 1 / v_dot * np.cross(b,c)
		bstar = 1 / v_dot * np.cross(a,c)
		cstar = 1 / v_dot * np.cross(a,b)
		recip_lattice.append(astar)
		recip_lattice.append(bstar)
		recip_lattice.append(cstar)
		recip_lattice = np.reshape(np.array(recip_lattice),([3,3]))


		num_C = atom_IDs[0]
		num_H = atom_IDs[1]
		num_O = atom_IDs[2]
		num_N = atom_IDs[3]
		num_S = atom_IDs[4]

		mass_C = 12.011
		mass_H = 1.008
		mass_O = 15.999
		mass_N = 14.007
		mass_S = 32.06

		##Some constants
		mass_kg = 1.66054e-27
		THz = 10e12
		bar = 1000
		Pa = 100000
		wvnum = 33.35641

		mass_tot = (num_C * mass_C + num_H * mass_H + num_O * mass_O + num_N * mass_N) * mass_kg #In kg

		density = mass_tot / volume
		
		C = np.reshape(ECs, (6,6))
		C = C * bar * Pa # Elastic constant matrix In Pa

		Christoffel_Mat = np.zeros([3,3])

		kpt_sampling_directions = np.array([0.5,0,0,
											0,0.5,0,
											0,0,0.5,
											0.5,0.5,0,
											0.5, 0, 0.5,
											0,0.5,0.5,
											0.5, 0.5, 0.5,
											0.5,-0.5,0,
											0.5,0,-0.5,
											0,0.5,-0.5,
											0.5,0.5,-0.5,
											0.5,-0.5,0.5,
											-0.5,0.5,0.5]).reshape([-1,3])

		Wmax = []
		for i in range(int(np.size(kpt_sampling_directions)/3)):
			d_cos = np.matmul(kpt_sampling_directions[i],recip_lattice)/ np.linalg.norm(np.matmul(kpt_sampling_directions[i], recip_lattice)) ##direction cosines with respect to reciprocal lattice
			
			#Building the Christoffel matrix, \Gamma_ik
			A_1 = np.square(d_cos[0])*C[0,0] + np.square(d_cos[1])*C[5,5] + np.square(d_cos[2])*C[4,4] + 2*d_cos[1]*d_cos[2]*C[4,5] + 2*d_cos[2]*d_cos[0]*C[0,4] + 2*d_cos[0]*d_cos[1]*C[0,5]
			A_2 = np.square(d_cos[0])*C[5,5] + np.square(d_cos[1])*C[1,1] + np.square(d_cos[2])*C[3,3] + 2*d_cos[1]*d_cos[2]*C[1,3] + 2*d_cos[2]*d_cos[0]*C[3,5] + 2*d_cos[0]*d_cos[1]*C[1,5]
			A_3 = np.square(d_cos[0])*C[4,4] + np.square(d_cos[1])*C[3,3] + np.square(d_cos[2])*C[2,2] + 2*d_cos[1]*d_cos[2]*C[2,3] + 2*d_cos[2]*d_cos[0]*C[2,4] + 2*d_cos[0]*d_cos[1]*C[3,4]
			alpha_23 = np.square(d_cos[0])*C[4,5] + np.square(d_cos[1])*C[1,3] + np.square(d_cos[2])*C[2,3] + 2*d_cos[1]*d_cos[2]*(1/2*(C[1,2] + C[3,3])) + 2*d_cos[2]*d_cos[0]*(1/2*(C[2,5]+C[3,4])) + 2*d_cos[0]*d_cos[1]*(1/2*(C[1,4]+C[3,5]))
			alpha_13 = np.square(d_cos[0])*C[0,4] + np.square(d_cos[1])*C[3,5] + np.square(d_cos[2])*C[2,4] + 2*d_cos[1]*d_cos[2]*(1/2*(C[2,5] + C[3,4])) + 2*d_cos[2]*d_cos[0]*(1/2*(C[0,2]+C[4,4])) + 2*d_cos[0]*d_cos[1]*(1/2*(C[0,3]+C[4,5]))
			alpha_12 = np.square(d_cos[0])*C[0,5] + np.square(d_cos[1])*C[1,5] + np.square(d_cos[2])*C[3,4] + 2*d_cos[1]*d_cos[2]*(1/2*(C[1,4] + C[3,5])) + 2*d_cos[2]*d_cos[0]*(1/2*(C[0,3]+C[4,5])) + 2*d_cos[0]*d_cos[1]*(1/2*(C[0,1]+C[5,5]))
			
			#Christoffel_Mat = np.zeros([3,3])
			Christoffel_Mat[0,0] = A_1
			Christoffel_Mat[0,1] = alpha_12
			Christoffel_Mat[0,2] = alpha_13
			Christoffel_Mat[1,0] = alpha_12
			Christoffel_Mat[1,1] = A_2
			Christoffel_Mat[1,2] = alpha_23
			Christoffel_Mat[2,0] = alpha_13
			Christoffel_Mat[2,1] = alpha_23
			Christoffel_Mat[2,2] = A_3
			
			PhaseVelocity = np.linalg.eig(Christoffel_Mat)[0] # = pv^2
			kzb = np.linalg.norm(d_cos) / 1E-10 # --- Length in pi / meter -- kzb 


			PhaseVelocity = np.transpose(PhaseVelocity)
			PhaseVelocity = np.sqrt(PhaseVelocity / density)
			PhaseVelocity = np.real(PhaseVelocity)
			nan_ind = np.isnan(PhaseVelocity)
			PhaseVelocity[nan_ind] = 0

			Coeff = 2 * PhaseVelocity * kzb / np.pi
			Coeff = Coeff / THz * wvnum
			Wmax = np.append(Wmax,Coeff)

		Wmax = Wmax.reshape([-1,3])

		#Using Wmax averages... not going this route
		#a_Wmax = Wmax[:,0]
		#b_Wmax = Wmax[:,1]
		#c_Wmax = Wmax[:,2]

		#a_Wmax = a_Wmax[a_Wmax!=0]
		#b_Wmax = b_Wmax[b_Wmax!=0]
		#c_Wmax = c_Wmax[c_Wmax!=0]

		#a_Wmax = np.average(a_Wmax)
		#b_Wmax = np.average(b_Wmax)
		#c_Wmax = np.average(c_Wmax)

		#avg_Wmax = np.array([a_Wmax, b_Wmax, c_Wmax])

		#Assign kpt direction in mesh sampling + Smoothing of the bands at kpt boundaries
		all_ids = []
		for i in range(np.size(mesh[:,0]) - 1):
			x1 = mesh[i,:]
			x2 = mesh[i+1,:]
			direction = x2 - x1
			dir_id = np.nonzero(direction)[0]
			direction2 = x1
			direction2[dir_id] = 0.5 #Edge of BZ to search for 13 "Base" kpt directions

			if np.linalg.norm(x2) == 0 and np.linalg.norm(x1) == 0:
				all_ids = np.append(all_ids,all_ids[-1])
			
			for j in range(np.size(kpt_sampling_directions[:,0])):
				bool_array = []
				for k in range(3):
					test = np.in1d(kpt_sampling_directions[j,k],direction2[k])
					bool_array = np.append(bool_array,test)
				if bool_array.all() == 1:
					all_ids = np.append(all_ids, j)

		Wmax_directions = []
		for i in range(np.size(all_ids) - 1):
			if i != 1:
				checkBack = all_ids[i-1]
				checkFront = all_ids[i+1]
				if all_ids[i] != checkBack and checkFront:
					all_ids[i] = checkFront
			Wmax_directions = np.append(Wmax_directions,Wmax[int(all_ids[i]),:])
		Wmax_directions = (Wmax_directions.reshape([-1,3]))
		Wmax2 = np.array((np.average(Wmax_directions[:,0]), np.average(Wmax_directions[:,1]), np.average(Wmax_directions[:,2])))
		
		#Add ID for endpoint
		all_ids = np.append(all_ids, all_ids[-1])

		TotKpts = np.size(all_ids)

		ss_freqs = ss_params.get_SS()[0]
		ss_freqs = np.reshape(ss_freqs,(-1, dim))
		DispFreqs = []
		mesh = ss_params.get_SS()[1]

		for i in range(TotKpts):
			x1 = mesh[i,:]
			dir_id = int(all_ids[i])
			y =  Wmax2 * abs(np.sin((np.linalg.norm(x1) / np.linalg.norm(kpt_sampling_directions[dir_id,:])) * np.pi / 2))

			DispFreqs = np.append(DispFreqs,y)

		DispFreqs = DispFreqs.reshape([-1,3])
		ss_freqs[:,0] = DispFreqs[:,0] / wvnum
		ss_freqs[:,1] = DispFreqs[:,1] / wvnum
		ss_freqs[:,2] = DispFreqs[:,2] / wvnum

		return ss_freqs

